//
// Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
// Ministerpräsidenten des Landes Schleswig-Holstein
// Staatskanzlei
// Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
//
// Lizenziert unter der EUPL, Version 1.2 oder - sobald
// diese von der Europäischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie dürfen dieses Werk ausschließlich gemäß
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
// https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
// ausdrücklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschränkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//

package mock

import (
	"context"
	"fmt"
	"github.com/stretchr/testify/assert"
	"google.golang.org/grpc"
	"google.golang.org/grpc/credentials/insecure"
	pb "info-manager-proxy/gen/go"
	"net"
	"testing"
	"time"
)

func TestStartGrpcServer(t *testing.T) {
	t.Run("should start gRPC server", func(t *testing.T) {
		SetUpGrpcServer()

		conn, err := net.DialTimeout("tcp", fmt.Sprintf("localhost:%d", GrpcMockPort), 2*time.Second)

		assert.NoError(t, err)

		conn.Close()
	})

	setUpGrpcEnv := func() (pb.InformationServiceClient, func()) {
		SetUpGrpcServer()

		conn, err := grpc.NewClient(fmt.Sprintf("localhost:%d", GrpcMockPort), grpc.WithTransportCredentials(insecure.NewCredentials()))
		assert.NoError(t, err)

		cleanup := func() {
			conn.Close()
		}
		client := pb.NewInformationServiceClient(conn)

		return client, cleanup
	}

	t.Run("TestGetInformation", func(t *testing.T) {
		t.Run("should have no error on get information with valid postfachId", func(t *testing.T) {
			client, cleanUp := setUpGrpcEnv()
			defer cleanUp()

			resp, err := client.GetInformation(context.Background(), &pb.GrpcInformationRequest{PostfachId: "testPostfachId"})

			assert.NoError(t, err)
			assert.NotNil(t, resp)
		})

		t.Run("should have error on get information with empty postfachId", func(t *testing.T) {
			client, cleanUp := setUpGrpcEnv()
			defer cleanUp()

			resp, err := client.GetInformation(context.Background(), &pb.GrpcInformationRequest{PostfachId: ""})

			assert.Error(t, err)
			assert.Nil(t, resp)
		})
	})

	t.Run("TestGetInformationById", func(t *testing.T) {
		t.Run("should have no error on get information by id", func(t *testing.T) {
			client, cleanUp := setUpGrpcEnv()
			defer cleanUp()

			resp, err := client.GetInformationById(context.Background(), &pb.GrpcInformationByIdRequest{Id: "testId"})

			assert.NoError(t, err)
			assert.NotNil(t, resp)
		})

		t.Run("should have error on get information by id with empty id", func(t *testing.T) {
			client, cleanUp := setUpGrpcEnv()
			defer cleanUp()

			resp, err := client.GetInformationById(context.Background(), &pb.GrpcInformationByIdRequest{Id: ""})

			assert.Error(t, err)
			assert.Nil(t, resp)
		})
	})
}
